' 
'    Copyright (c) 2012 Microsoft Corporation. All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code. For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see all Code Samples for Windows Phone, visit http://go.microsoft.com/fwlink/?LinkID=219604 
'  
'

Imports Microsoft.Phone.Controls
Imports System.Windows.Controls
Imports System.Windows
Imports System.IO.IsolatedStorage
Imports System.IO
Imports System.Collections.ObjectModel

Namespace HelperFunctions
	Public NotInheritable Class Utilities
		' Saves the current list of memos to iso-storage

		Public Shared Sub SaveCollectionToIsoStorage(ByVal fileName As String, ByVal collection As IList(Of String))
			If collection.Count = 0 Then
				Dim isoStore As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
				isoStore.DeleteFile(fileName)
			Else
				WriteCollectionToIsoStorage(fileName, collection)
			End If
		End Sub

		' Writes all entries in a collection to a file, one line at a time.
		Public Shared Sub WriteCollectionToIsoStorage(ByVal fileName As String, ByVal outputLines As IList(Of String))
			If outputLines.Count < 1 Then
				Exit Sub
			End If

			Dim isoStore As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication

            Using isoFileStream = New IsolatedStorageFileStream(fileName, FileMode.OpenOrCreate Or FileMode.Truncate, isoStore)
                ' Write the data
                Using isoFileWriter = New StreamWriter(isoFileStream)
                    For i = 0 To outputLines.Count - 1
                        isoFileWriter.WriteLine(outputLines(i))
                    Next i
                End Using
                isoFileStream.Close()
            End Using
		End Sub

		' Reads contents of file at 'fileName' one line at a time and puts them in collection 'textLines'
		Public Shared Sub ReadLinesFromIsoStorageFile(ByVal fileName As String, ByVal textLines As IList(Of String))
			Dim isoStore As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication

			' Read text lines from specified file and put into the string collection
			If isoStore IsNot Nothing AndAlso isoStore.FileExists(fileName) Then
				Using memoReader As New StreamReader(isoStore.OpenFile(fileName, FileMode.Open, FileAccess.Read))
					textLines.Clear()

					Dim index As Integer = 1
					Do While Not memoReader.EndOfStream
						textLines.Add(memoReader.ReadLine)
                        index = index + 1
					Loop
					memoReader.Close()
				End Using
			End If
		End Sub

		' Appends one line of text to a file in iso-storage. It does not have acccess to a Dispatcher so be sure to call it on a UI thread as necessary.
		Public Shared Sub WriteLineToIsoStorage(ByVal fileName As String, ByVal outputLine As String)
			Try
				If String.IsNullOrEmpty(outputLine) Then
					Exit Sub
				End If

				Dim isoStore As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication

				Using isoFileStream = New IsolatedStorageFileStream(fileName, FileMode.OpenOrCreate Or FileMode.Append, isoStore)
					' Write the data
					Using isoFileWriter = New StreamWriter(isoFileStream)
						isoFileWriter.WriteLine(outputLine)
					End Using
					isoFileStream.Close()
				End Using
			Catch ex As Exception
				MessageBox.Show("Exception in WriteLineToIsoStorage: " & ex.Message)
			End Try
		End Sub

		' Simple helper function to set TextBox text on the UI thread
		Public Shared Sub SetTextBoxTextSafely(ByVal tBox As TextBox, ByVal text As String)
			tBox.Dispatcher.BeginInvoke(Sub() tBox.Text = text)
		End Sub

		' Return a string list array of the positive integers "1" through limit.
		Public Shared Function GetStringListOfPositiveIntegers(ByVal limit As Integer) As List(Of String)
			Dim stringList As New List(Of String)(If(limit < 0, 0, limit))

			For i  = 1 To limit
				stringList.Add(i.ToString)
			Next i

			Return stringList
		End Function
	End Class
End Namespace

