Imports HelperFunctions

' 
'    Copyright (c) 2012 Microsoft Corporation. All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code. For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see all Code Samples for Windows Phone, visit http://go.microsoft.com/fwlink/?LinkID=219604 
'  
'

'#define SemanticsDebug

Imports System.Collections.ObjectModel
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports Microsoft.Phone.Controls
Imports Windows.Phone.Speech.Recognition
Imports Windows.Phone.Speech.VoiceCommands

' Code behind for the Magic Memo sample code ViewMemos page. This sample uses simple handlers and helper functions to illustrate the use of speech recognition and synthesis
' in Window Phone 8. It does not implement MVVM and does not store static strings in resources as a globalized application would.
Namespace MagicMemo
	Partial Public Class ViewMemos
		Inherits PhoneApplicationPage

		'
		' Code behind for View Memos page for the Magic Memo sample application View Memos page
		'

		#Region "Speech objects"
		' Recognizer for commanding the page. Uses an SRGS grammar.
		Private commandReco As SpeechRecognizerUI
		#End Region

		#Region "Private fields"
		' Observable collection for storing the current set of memos
		Private memoList As New ObservableCollection(Of String)

		' File in isostorage to store memos
		Private fileName As String = "savedMemos.txt"
		#End Region

		' Constructor for this page
		Public Sub New()
			InitializeComponent() ' Autogenerated
			InitializeSpeech()
		End Sub

		#Region "EventHandlers"
		' Handler for Loaded event.
		Private Sub ViewMemosPage_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Read saved memos from iso-storage.
			Utilities.ReadLinesFromIsoStorageFile(fileName, memoList) ' In HelperFunctions.vb

			' Set the listbox source to the collection of memos loaded from iso-storage.
			MemosLB.ItemsSource = memoList

			UpdateNumberPhraseList("num", memoList.Count, "MagicMemoEnu")

			' Handle the case where page was launched directly by Voice Command
			If Me.NavigationContext.QueryString IsNot Nothing AndAlso Me.NavigationContext.QueryString.ContainsKey("voiceCommandName") Then
				' If in this block then page was launched by Voice Command

				' Take appropriate action depending on the voice command
				Dim commandName As String = NavigationContext.QueryString("voiceCommandName")
				Dim spokenNumber As String = ""

				If commandName = "showOne" AndAlso Me.NavigationContext.QueryString.TryGetValue("num", spokenNumber) Then
					' Command was "Show memo number 'num'".
					Dim index As Integer = -1

					If Integer.TryParse(spokenNumber, index) AndAlso index <= memoList.Count AndAlso index > 0 Then
						Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show(String.Format("Memo {0}:  ""{1}""", index, memoList(index - 1))))
					Else
						' This shouldn't happen after first run unless there are no saved memos, because UpdateNumberPhraseList will have set the 
						' 'num' PhraseList to be equal to the number of saved memos.
						Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show(String.Format("No memo {0} yet", index + 1)))
					End If
				End If
				' Note: 
				' There is no need for an "else" block in this application because if launched by another VoiceCommand 
				' then commandName="showAll" so the only action is to display the page. An application that has many voice commands 
				' associated with a page could use something like a switch block on the commandName.
			End If
		End Sub

		' Handler for 'Clear all' button tap
		Private Sub ClearAllBT_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			' Remove all memos
			memoList.Clear()
		End Sub

		' Handler for 'Clear one' button tap
		Private Sub ClearOneBT_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			' Remove the selected memo 
			If MemosLB.SelectedIndex > -1 Then
				memoList.RemoveAt(MemosLB.SelectedIndex)
			End If
		End Sub

		' Handler for 'Save' button tap
		Private Sub SaveBT_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			' Save changes to the iso-storage file.
			Utilities.SaveCollectionToIsoStorage(fileName, memoList)

			' Update the num phrase list to match the current number of saved memos.
			UpdateNumberPhraseList("num", memoList.Count, "MagicMemoEnu")
		End Sub

		' Handler for ListBox 'Selection changed'
		Private Sub MemosLB_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			' Display the number of the selected memo in the TextBox or blank if no selection
			If MemosLB.SelectedIndex < 0 Then
				Utilities.SetTextBoxTextSafely(SelectedIndexTB, "")
			Else
				Utilities.SetTextBoxTextSafely(SelectedIndexTB, String.Format("{0}", MemosLB.SelectedIndex + 1))
			End If
		End Sub

		' Handler for 'Enter New' button tap
		Private Sub EnterNewBT_Tap(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Go to main page to enter a new memo
			NavigationService.Navigate(New Uri("/MainPage.xaml", UriKind.Relative))
		End Sub

		' Handler for mic tap
		' Launches a speech recognition session using the commandReco object, which already has the SRGS grammar added
		' and pre-loaded (see InitializeSpeech helper function). This case uses the 'await' construct to get the recognition
		' result rather than an explicit callback function, and executes commands based on the result.
		Private async Sub micImage_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			Dim commandResult = await commandReco.RecognizeWithUIAsync

			If commandResult.ResultStatus = SpeechRecognitionUIStatus.Succeeded AndAlso commandResult.RecognitionResult IsNot Nothing AndAlso commandResult.RecognitionResult.Semantics IsNot Nothing Then
				' Execute commands based on the semantic values returned rather than the actual utterance recognized. Using
				' semantics has the advantage that there's no need to parse the actual utterance to discern what the user 
				' intended. The code acts on what the user intended, not what he/she actually said.
				' The semantic values are defined in the SRGS grammar <tag> elements using a subset of ECMAscript. see
				' http://www.w3.org/TR/semantic-interpretation/ for more information.
				Dim commandSemantics = commandResult.RecognitionResult.Semantics

#If SemanticsDebug Then ' Displays the semantics returned from the grammar for debugging.
				Dim kv As String = "Semantics: "

				For Each k As String In commandSemantics.Keys
					kv &= String.Format(" {0} = {1}; ", k, TryCast(commandSemantics(k).Value, String))
				Next k

			   Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show(kv))
#End If

				Dim process As SemanticProperty = Nothing

				If commandSemantics.TryGetValue("process", process) Then
					Dim numSemantic As SemanticProperty = Nothing
					Dim scopeSemantic As SemanticProperty = Nothing
					Dim selectedIndex As Integer = 0

					' In general a semantic value can be any object, but in this case it's a string
					Select Case TryCast(process.Value, String)
						' For this grammar the 'process' semantic more or less corresponds to a button on the page.
						Case "select"

							' The command was "Select memo number 'num'".
							If commandSemantics.TryGetValue("num", numSemantic) AndAlso Integer.TryParse(TryCast(numSemantic.Value, String), selectedIndex) AndAlso selectedIndex > 0 AndAlso selectedIndex <= memoList.Count Then
								MemosLB.SelectedIndex = selectedIndex - 1
							End If

						Case "clear"
							' Command was "Clear memo number 'num'", "Clear all" or "Clear Selected".

							If commandSemantics.TryGetValue("num", numSemantic) Then
								' The command was "Clear memo number 'num'".
								If Integer.TryParse(TryCast(numSemantic.Value, String), selectedIndex) Then
									selectedIndex -= 1

									If selectedIndex < memoList.Count AndAlso selectedIndex >= 0 Then
										memoList.RemoveAt(selectedIndex)
									End If
								End If
							ElseIf commandSemantics.TryGetValue("scope", scopeSemantic) Then
								If TryCast(scopeSemantic.Value, String) = "selected" Then
									' The command was "Clear selected"
									If MemosLB.SelectedIndex > -1 Then
										memoList.RemoveAt(MemosLB.SelectedIndex)
									End If
								ElseIf TryCast(scopeSemantic.Value, String) = "all" Then
									' The command was "Clear all"
									memoList.Clear()
								End If
							End If

						Case "save"
							' Command was "Save" or "Save Changes"

							' Save current memo collection
							Utilities.SaveCollectionToIsoStorage(fileName, memoList)
							UpdateNumberPhraseList("num", memoList.Count, "MagicMemoEnu")

						Case "new"
							' Command was "Enter new" 

							' Navigate to MainPage to enter a new memo
							NavigationService.Navigate(New Uri("/MainPage.xaml", UriKind.Relative))

						Case Else
					End Select
				End If
			End If
		End Sub

		' Display the memoList index on the page, referenced from a textblock in ViewMemos.xaml. 
		' This is kind of a hack. Could possibly use a Binding Converter instead.
		Private Sub TextBlock_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			Dim tb As TextBlock = TryCast(sender, TextBlock)
			Dim value As String = TryCast(tb.DataContext, String)
			tb.Text = (memoList.IndexOf(value) + 1).ToString
		End Sub
		#End Region

		#Region "Private speech functions"
		' Sets up the commandReco speech recognition object and preloads the grammar
		Private async Sub InitializeSpeech()
			If commandReco Is Nothing Then
				Try
					' Instantiate speech recognizer
					commandReco = New SpeechRecognizerUI

					' Set prompt spoken to user
					commandReco.Settings.ListenText = "Say selection or button name"

					' Set text displayed as examples
					commandReco.Settings.ExampleText = "Ex. 'Clear three' or 'Select two'"

					' Path to SRGS grammar to recognize button names and numbers
					Dim grammarFileUri As New Uri("ms-appx:///ViewMemos.grxml")

					' Adds grammar but does not load it
					commandReco.Recognizer.Grammars.AddGrammarFromUri("srgsCommands", grammarFileUri)

					' Preloads the grammar to reduce latency during recognition. This step is optional. If not done the call to 
					' RecognizeAsyncUI will do it but recognition may take more time.
					await commandReco.Recognizer.PreloadGrammarsAsync
				Catch ex As Exception
					Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("Exception in commandReco initialization" & ex.Message))
				End Try
			End If
		End Sub

		' Updates the "num" PhraseList to have the same number of entries as the number of saved memos.
		' This supports "Magic Memo show memo five" if there are five or more memos saved.
		Private async Sub UpdateNumberPhraseList(ByVal phraseList As String, ByVal newLimit As Integer, ByVal commandSetName As String)
			Dim positiveIntegers As List(Of String) = Utilities.GetStringListOfPositiveIntegers(Math.Max(1, newLimit))

			Try
				Dim vcs As VoiceCommandSet = Nothing

				If VoiceCommandService.InstalledCommandSets.TryGetValue(commandSetName, vcs) Then
					' Update phrase list to the new numbers
					await vcs.UpdatePhraseListAsync(phraseList, positiveIntegers)
				End If
			Catch ex As Exception
				Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("Exception in UpdateNumberPhraseList " & ex.Message))
			End Try
		End Sub
		#End Region

	End Class
End Namespace
